from django.db.models.fields.related import ForeignKey, RelatedField
from djeu.forms.widgets import ExtendedModelMultipleChoiceField


class ExtendedForeignKey(ForeignKey):

    def __init__(self, to, on_delete, related_name=None, related_query_name=None,
                 limit_choices_to=None, parent_link=False, to_field=None,
                 db_constraint=True, depends=None, **kwargs):
        super(ExtendedForeignKey, self).__init__(to, on_delete, related_name, related_query_name,
                                                 limit_choices_to, parent_link, to_field, db_constraint,
                                                 **kwargs)
        self.depends = depends



from django import forms
from django.conf import settings
from .date_picker_input import DatePickerInput


class JulianDatePickerInput(DatePickerInput):

    @property
    def media(self):
        extra = '' if settings.DEBUG else '.min'
        return forms.Media(
            js=(
                'admin/js/vendor/jquery/jquery%s.js' % extra,
                'jquery-ui/jquery-ui%s.js' % extra,
                # 'djeu/js/jquery-datepicker.js',
                'djeu/js/i18n/datepicker-ru-RU-julian.js',
                'djeu/js/jquery-datepicker.init.js',
            ) + (
                'admin/js/jquery.init.js',
            ),
            css={
                'screen': (
                    'jquery-ui/jquery-ui.min.css',
                ),
            },
        )



from .linked_file_input import *
from .date_picker_input import *
from .julian_date_picker_input import *
from .extended_autocomplete_select_multiply import *
from .extended_model_multiple_choice_field import *
from .pdf_viewer_input import *


import os

SITE_ID = 1

BASE_DIR = os.path.dirname(__file__)

ROOT_URLCONF = 'urls'
SECRET_KEY = 'secretkey'
SITE_ROOT = os.path.dirname(os.path.abspath(__file__))

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.sessions',
    'django.contrib.contenttypes',
    'django.contrib.admin',
    'django.contrib.messages',
    'django.contrib.sites',
    'crispy_forms',
    'contact_form'
)

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'test.db',                      # Or path to database file if using sqlite3.
        'USER': '',                      # Not used with sqlite3.
        'PASSWORD': '',                  # Not used with sqlite3.
        'HOST': '',                      # Set to empty string for localhost. Not used with sqlite3.
        'PORT': '',                      # Set to empty string for default. Not used with sqlite3.
    }
}

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware'
)

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.contrib.auth.context_processors.auth',
                'django.template.context_processors.debug',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'django.template.context_processors.tz',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

CRISPY_TEMPLATE_PACK = 'uni_form'



from django.db.models import Model
from django.db.models.fields.related import ManyToManyField, RelatedField
from djeu.forms.widgets import ExtendedModelMultipleChoiceField


class ExtendedManyToManyField(ManyToManyField):

    def __init__(self, to, related_name=None, related_query_name=None,
                 limit_choices_to=None, symmetrical=None, through=None,
                 through_fields=None, db_constraint=True, db_table=None,
                 swappable=True, key_data_components=None, additional_fields=None,
                 **kwargs):
        super().__init__(to, related_name, related_query_name, limit_choices_to,
                         symmetrical, through, through_fields, db_constraint, db_table,
                         swappable, **kwargs)
        self.through = through
        self.key_data_components = key_data_components if key_data_components else tuple(
            [attname for attname in through_fields if attname != self.remote_field.name]) if through_fields else []
        self.additional_fields = additional_fields

    # def _limit_choices_to(self):
    #     if hasattr(super(ExtendedManyToManyField, self), '_limit_choices_to') and super(ExtendedManyToManyField, self)._limit_choices_to:
    #         return super(ExtendedManyToManyField, self)._limit_choices_to()
    #     if self.through:
    #         return self.through.objects.all()
    #     return self.remote_field.model.objects.all()

    def value_from_object(self, obj):
        if self.through:
            through_attname = self.remote_field.name
            return [] if obj.pk is None else list(self.through.objects.filter(**{through_attname: obj}))
        return super(ExtendedManyToManyField, self).value_from_object(obj)

    def formfield(self, *, using=None, **kwargs):
        # self.remote_field.field.objects.all()
        # self.remote_field._default_manager.using(using)
        queryset = self.through.objects.all(
        ) if self.through else self.remote_field.model._default_manager.using(using)

        defaults = {
            'form_class': ExtendedModelMultipleChoiceField,
            'queryset': queryset,
            **kwargs,
        }
        # If initial is passed in, it's a list of related objects, but the
        # MultipleChoiceField takes a list of IDs.
        if defaults.get('initial') is not None:
            initial = defaults['initial']
            if callable(initial):
                initial = initial()
            defaults['initial'] = [i.pk for i in initial]
        return super(RelatedField, self).formfield(**defaults)

    def get_prep_value(self, value):
        return super(ExtendedManyToManyField, self).get_prep_value(value)

    def add(self, obj):
        return super(ExtendedManyToManyField, self).add(obj)

    def __str__(self):
        if self.through:
            # self.through.objects
            return ', '.join(self.all())

        return super(ExtendedManyToManyField, self).__str__()

    def __repr__(self):
        self.__str__()
